package com.ruoyi.ss.bonus.service;

import com.ruoyi.common.domain.vo.CommonCountVO;
import com.ruoyi.common.domain.vo.CommonSumVO;
import com.ruoyi.common.domain.vo.LongDecimalVO;
import com.ruoyi.ss.bonus.domain.Bonus;
import com.ruoyi.ss.bonus.domain.BonusProvideQuery;
import com.ruoyi.ss.bonus.domain.BonusQuery;
import com.ruoyi.ss.bonus.domain.BonusVO;
import com.ruoyi.ss.bonus.domain.vo.BonusDailyAmountVO;
import com.ruoyi.ss.bonus.domain.vo.BonusMonthAmountVO;
import com.ruoyi.ss.bonus.domain.vo.ProvideBonusVO;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;

/**
 * 分成明细Service接口
 *
 * @author ruoyi
 * @date 2024-08-20
 */
public interface BonusService
{
    /**
     * 查询分成明细
     *
     * @param id 分成明细主键
     * @return 分成明细
     */
    public BonusVO selectBonusById(Long id);

    /**
     * 查询分成明细列表
     *
     * @param bonus 分成明细
     * @return 分成明细集合
     */
    public List<BonusVO> selectBonusList(BonusQuery bonus);

    /**
     * 新增分成明细
     *
     * @param bonus 分成明细
     * @return 结果
     */
    public int insertBonus(Bonus bonus);

    /**
     * 修改分成明细
     *
     * @param bonus 分成明细
     * @return 结果
     */
    public int updateBonus(Bonus bonus);

    /**
     * 批量删除分成明细
     *
     * @param ids 需要删除的分成明细主键集合
     * @return 结果
     */
    public int deleteBonusByIds(Long[] ids);

    /**
     * 删除分成明细信息
     *
     * @param id 分成明细主键
     * @return 结果
     */
    public int deleteBonusById(Long id);

    /**
     * 批量新增
     */
    int batchInsert(List<Bonus> bonusList);

    /**
     * 处理分成支付
     *
     * @param bonusList 分成列表
     * @return 处理数量
     */
    int payBonus(List<BonusVO> bonusList);

    /**
     * 支付分成
     * @param bonus
     */
    int payBonus(BonusVO bonus);

    /**
     * 处理分成，按照比例分出金额
     * @param bonusList 分成列表
     * @param money 总金额
     */
    int partBonus(List<BonusVO> bonusList, BigDecimal money);

    /**
     * 根据到账方统计
     */
    List<CommonCountVO<Long>> selectCountByArrival(BonusQuery query);

    /**
     * 根据到账方统计订单金额
     */
    List<CommonSumVO<Long>> selectBillAmountByArrival(BonusQuery query);

    /**
     * 查询提供分成
     */
    List<ProvideBonusVO> selectProvideBonus(BonusProvideQuery query);

    /**
     * 当未分成时退款
     */
    int refundWhenWaitDivide(Long id, BigDecimal amount);

    /**
     * 当已分成时退款
     */
    int refundWhenDividend(Long id, BigDecimal amount);

    /**
     * 数据隔离过滤
     */
    <T extends Bonus> List<T> filterBonusScope(List<T> bonusList);

    /**
     * 按月查询分成金额
     */
    List<BonusMonthAmountVO> selectMonthAmount(BonusQuery query);

    /**
     * 按日查询分成
     */
    List<BonusDailyAmountVO> selectDailyAmount(BonusQuery query);

    /**
     * 查询数量
     * @param query
     * @return
     */
    int selectCount(BonusQuery query);

    /**
     * 支付预计分成在指定时间的分成
     */
    int payBonusBeforeTime(LocalDateTime time);

    /**
     * 查询待分成金额，以到账方分组统计
     */
    List<LongDecimalVO> selectWaitAmountGroupByArrival(BonusQuery query);
}
